﻿using Orchard;
using Orchard.ContentManagement;
using Orchard.Data;
using Orchard.DisplayManagement;
using Orchard.Localization;
using Orchard.Mvc;
using Orchard.Settings;
using Orchard.Themes;
using Orchard.UI.Navigation;
using Orchard.Logging;
using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using System.Web.Helpers;
using Newtonsoft.Json.Linq;
using @(Model.Module.ModuleName).Core;
using @(Model.Module.ModuleName).Models;
using @(Model.Module.ModuleName).Interfaces;
using @(Model.Module.ModuleName).ServiceModels;
using @(Model.Module.ModuleName).ViewModels;

namespace @(Model.Module.ModuleName).Controllers
{
    [Themed]
    public class @(Model.TypeName)Controller : Controller
    {
        private readonly I@(Model.TypeName)Service _@(Model.TypeName)Service;
        private readonly IContentManager _contentManager;
        private readonly ITransactionManager _transactionManager;
        private readonly ISiteService _siteService;

        public @(Model.TypeName)Controller(
            IOrchardServices services,
            I@(Model.TypeName)Service @(Model.TypeName)Service,
            IContentManager contentManager,
            ITransactionManager transactionManager,
            ISiteService siteService,
            IShapeFactory shapeFactory)
        {
            Services = services;
            _@(Model.TypeName)Service = @(Model.TypeName)Service;
            _contentManager = contentManager;
            _transactionManager = transactionManager;
            _siteService = siteService;
            T = NullLocalizer.Instance;
            Logger = NullLogger.Instance;
            Shape = shapeFactory;
        }


        dynamic Shape { get; set; }
        public Localizer T { get; set; }
        public ILogger Logger { get; set; }
        public IOrchardServices Services { get; set; }

        protected ActionResult ShapePartialResult(dynamic shape)
        {
            return new ShapePartialResult(this, shape);
        }

        protected JsonResult Jsonp(object data, JsonRequestBehavior behavior = JsonRequestBehavior.AllowGet)
        {
            if (this.IsIE() && this.AcceptText())
            {
                return Json(data, "text/html", behavior);
            }
            else
            {
                return Json(data, behavior);
            }
        }

        private ActionResult SuccessResult()
        {
            string p = Request.Unvalidated()["p"];
            if (p == "json")
            {
                return Jsonp("OK");
            }
            else
            {
                string returnUrl = Request.Unvalidated()["returnUrl"];
                if (!string.IsNullOrEmpty(returnUrl))
                {
                    return Redirect(returnUrl);
                }
                else
                {
                    return Content("OK");
                }
            }
        }

        private @(Model.TypeName)ViewModel GetViewModel(@(Model.TypeName)ViewModel viewModel, PagerParameters pagerParameters)
        {
            var entities = _@(Model.TypeName)Service.Get@(Model.TypeName)List(viewModel.Search, viewModel.Sort, true);
            int totalItemCount = 0;
            if (entities != null)
            {
                totalItemCount = entities.Count();
            }

            Pager pager = new Pager(_siteService.GetSiteSettings(), pagerParameters);

            var model = new @(Model.TypeName)ViewModel
            {
                Search = viewModel.Search,
                Sort = viewModel.Sort ?? new @(Model.TypeName)Sort(),
                Entities = entities != null ? entities.Skip(pager.GetStartIndex()).Take(pager.PageSize) : null,
                Pager = Shape.Pager(pager)
                    .TotalItemCount(totalItemCount)
                    .RouteData(new RouteValueDictionary  
                        {
                            {"action", "Table"}
                        }),
            };
            model.Table = Shape.DisplayTemplate(TemplateName: "@(Model.TypeName)/Table", Model: model);
            model.SearchDialog = Shape.EditorTemplate(TemplateName: "@(Model.TypeName)/Search", Model: model);

            return model;
        }

        private dynamic GetEditor(@(Model.TypeName)Model model)
        {
            return Shape.EditorTemplate(TemplateName: "@(Model.TypeName)/Edit", Model: model);
        }

        public ActionResult Table(@(Model.TypeName)ViewModel viewModel, PagerParameters pagerParameters)
        {
            if (!Services.Authorizer.Authorize(Permissions.Manage@(Model.TypeName), T("Not allowed to manage @(Model.TypeName)")))
                return new HttpUnauthorizedResult();

            var model = GetViewModel(viewModel, pagerParameters);
            return ShapePartialResult(model.Table);
        }

        public ActionResult List(@(Model.TypeName)ViewModel viewModel, PagerParameters pagerParameters)
        {
            if (!Services.Authorizer.Authorize(Permissions.Manage@(Model.TypeName), T("Not allowed to manage @(Model.TypeName)")))
                return new HttpUnauthorizedResult();

            var model = GetViewModel(viewModel, pagerParameters);
            return View(model);
        }

        [HttpGet]
        public ActionResult Create()
        {
            if (!Services.Authorizer.Authorize(Permissions.Manage@(Model.TypeName), T("Not allowed to manage @(Model.TypeName)")))
                return new HttpUnauthorizedResult();

            @(Model.TypeName)Model model = new @(Model.TypeName)Model();
            return ShapePartialResult(GetEditor(model));
        }

        [HttpGet]
        public ActionResult Edit(int id)
        {
            if (!Services.Authorizer.Authorize(Permissions.Manage@(Model.TypeName), T("Not allowed to manage @(Model.TypeName)")))
                return new HttpUnauthorizedResult();

            @(Model.TypeName)Model model = _@(Model.TypeName)Service.Get@(Model.TypeName)(id);
            return ShapePartialResult(GetEditor(model));
        }


        [HttpPost]
        public ActionResult Update(@(Model.TypeName)Model model, HttpPostedFileBase Photo1File)
        {
            if (!Services.Authorizer.Authorize(Permissions.Manage@(Model.TypeName), T("Not allowed to manage @(Model.TypeName)")))
                return new HttpUnauthorizedResult();

            if (!ModelState.IsValid)
            {
                return ShapePartialResult(GetEditor(model));
            }
            else
            {
                try
                {
                    int result = 0;

                    if (model.Id <= 0)
                    {
                        result = _@(Model.TypeName)Service.Create(model);
                    }
                    else
                    {
                        result = _@(Model.TypeName)Service.Update(model);
                    }

                    if (result > 0)
                    {
                        return SuccessResult();
                    }
                }
                catch (Exception ex)
                {
                    Logger.Error(ex, "@(Model.TypeName) Update failed.");
                }
            }

            return ShapePartialResult(GetEditor(model));
        }

        public ActionResult Delete(int id)
        {
            if (!Services.Authorizer.Authorize(Permissions.Manage@(Model.TypeName), T("Not allowed to manage @(Model.TypeName)")))
                return new HttpUnauthorizedResult();

            int result = _@(Model.TypeName)Service.Delete(id);
            if (result > 0)
            {
                return SuccessResult();
            }
            else
            {
                return new EmptyResult();
            }
        }
    }
}