﻿using @(Model.Module.ModuleName).Common;
using @(Model.Module.ModuleName).Interfaces;
using @(Model.Module.ModuleName).Models;
using @(Model.Module.ModuleName).ServiceModels;
using @(Model.Module.ModuleName).ViewModels;
using Orchard;
using Orchard.Caching;
using Orchard.ContentManagement;
using Orchard.Localization;
using Orchard.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Newtonsoft.Json.Linq;
using Orchard.Data;
using RaisingStudio.Contents.RepositoryFactory.Services;

namespace @(Model.Module.ModuleName).Services
{
    public class @(Model.TypeName)Service : I@(Model.TypeName)Service
    {
        private const string SignalName = CacheAndSignals.@(Model.TypeName)Signal;
        private const string CacheName = CacheAndSignals.@(Model.TypeName)Cache;

        public const string @(Model.TypeName)ContentTypeName = "@(Model.TypeName)";

        private readonly IContentManager _contentManager;
        private readonly ICacheManager _cacheManager;
        private readonly ISignals _signals;
        private readonly IOrchardServices _orchardServices;

        public @(Model.TypeName)Service(
            IContentManager contentManager,
            ICacheManager cacheManager,
            ISignals signals,
            IOrchardServices orchardServices,
            IContentsRepositoryFactory repositoryFactory)
        {
            _contentManager = contentManager;
            _orchardServices = orchardServices;
            _cacheManager = cacheManager;
            _signals = signals;

            SetupRepository(repositoryFactory.GetRepository<@(Model.TypeName)PartRecord>(@(Model.TypeName)ContentTypeName));

            T = NullLocalizer.Instance;
            Logger = NullLogger.Instance;
        }

        public Localizer T { get; set; }
        public ILogger Logger { get; set; }

        private IRepository<@(Model.TypeName)PartRecord> _@(Model.TypeName)Repository;
        public void SetupRepository(
            IRepository<@(Model.TypeName)PartRecord> @(Model.TypeName)Repository)
        {
            _@(Model.TypeName)Repository = @(Model.TypeName)Repository;
        }


        private void MonitorSignal(AcquireContext<string> ctx)
        {
            ctx.Monitor(_signals.When(SignalName));
        }

        public void TriggerSignal()
        {
            _signals.Trigger(SignalName);
        }


        private @(Model.TypeName)Model Convert(@(Model.TypeName)PartRecord record)
        {
            if (record != null)
            {
                return new @(Model.TypeName)Model
                {
                    Id = record.Id,
                    @foreach(var property in Model.Properties)
                    {
@:                    @(property.PropertyName) = record.@(property.PropertyName),
                    }
                };
            }
            return null;
        }

        private @(Model.TypeName)PartRecord Convert(@(Model.TypeName)Model model, @(Model.TypeName)PartRecord record)
        {
            if (model != null && record != null)
            {
              @foreach(var property in Model.Properties)
              {
@:              record.@(property.PropertyName) = model.@(property.PropertyName);
              }
              return record;
            }
            return null;
        }


        private IEnumerable<@(Model.TypeName)Model> InternalGet@(Model.TypeName)List()
        {
            return from r in _@(Model.TypeName)Repository.Table
                    select Convert(r);
        }

        private IEnumerable<@(Model.TypeName)Model> InternalGet@(Model.TypeName)List(bool allowCache)
        {
            try
            {
                if (!allowCache)
                {
                    return InternalGet@(Model.TypeName)List();
                }
                else
                {
                    return _cacheManager.Get(CacheName, ctx =>
                    {
                        MonitorSignal(ctx);
                        return InternalGet@(Model.TypeName)List().ToList();
                    });
                }
            }
            catch (Exception ex)
            {
                Logger.Error(ex, "Get @(Model.TypeName) List failed.");
            }
            return null;
        }

        public IEnumerable<@(Model.TypeName)Model> Get@(Model.TypeName)List(@(Model.TypeName)Search search = null, @(Model.TypeName)Sort sort = null, bool allowCache = true)
        {

            try
            {
                var q = InternalGet@(Model.TypeName)List(allowCache);

                #region Search
                if (search != null)
                {
                    q = q.Where
                        (
                            p =>
                                (search.Id == null || p.Id == search.Id)
                        );
                }
                #endregion
                #region Sort
                if (sort != null)
                {
                    if (sort.Id != null)
                    {
                        q = (sort.Id == OrderByDirection.Ascending) ? q.OrderBy(t => t.Id) : q.OrderByDescending(t => t.Id);
                    }
                }
                #endregion
                return q;
            }
            catch (Exception ex)
            {
                Logger.Error(ex, "Get @(Model.TypeName) List failed.");
            }
            return null;
        }

        public @(Model.TypeName)Model Get@(Model.TypeName)(int id, bool allowCache = true)
        {
            var q = InternalGet@(Model.TypeName)List(allowCache);
            return q.SingleOrDefault(p => p.Id == id);
        }


        public int Create(@(Model.TypeName)Model entity)
        {
            try
            {
                if (entity != null)
                {
                    var record = new @(Model.TypeName)PartRecord();
                    _@(Model.TypeName)Repository.Create(Convert(entity, record));
                    int id = record.Id;
                    if (id > 0)
                    {
                        TriggerSignal();
                    }
                    return id;
                }
            }
            catch (Exception ex)
            {
                Logger.Error(ex, "Create @(Model.TypeName) failed, entity: {0}", JObject.FromObject(entity));
            }
            return -1;
        }

        public int Update(@(Model.TypeName)Model entity)
        {
            try
            {
                if (entity != null)
                {
                    int id = entity.Id;
                    var record = _@(Model.TypeName)Repository.Get(id);
                    _@(Model.TypeName)Repository.Update(Convert(entity, record));
                    TriggerSignal();
                    return id;
                }

            }
            catch (Exception ex)
            {
                Logger.Error(ex, "Update @(Model.TypeName) failed, entity: {0}", JObject.FromObject(entity));
            }
            return -1;
        }

        public int Delete(int id)
        {
            try
            {
                var record = _@(Model.TypeName)Repository.Get(id);
                if (record != null)
                {
                    _@(Model.TypeName)Repository.Delete(record);
                    TriggerSignal();
                    return id;
                }
            }
            catch (Exception ex)
            {
                Logger.Error(ex, "Delete @(Model.TypeName) failed, Id: {0}", id);
            }
            return -1;
        }
    }
}